from typing import List
from getpass_asterisk.getpass_asterisk import getpass_asterisk

class Terminal:
    def __init__(self, prompt = ">> "):
        self.prompt = prompt

    def request(self, issue):
        return input(issue + self.prompt)

    def request_password(self, issue, repeat=False):
        while repeat:
            password = getpass_asterisk(issue + self.prompt)
            if getpass_asterisk("enter passphrase again" + self.prompt) == password:
                return password
            else: 
                print("passphrases didn't match")
                print("try again")
        else:
            return getpass_asterisk(issue + self.prompt)

    def choice(self, issue):
        while True:
            check = input(issue + "[y/n] ")
            if check.lower() == "y":
                return True
            elif check.lower() == "n":
                return False

            print("please enter one letter ('y' or 'n')")            

    def select(self, options: List[str], issue = ""):
        if issue:
            print("C: " + issue)

        for i in range(0, len(options)):
            print("{number}: {option}".format(number=i + 1, option = options[i]))

        while True:
            try:
                selection = int(input(self.prompt))
            except ValueError:
                print("please, enter a number")                
            else:
                if 1 <= selection <= len(options):
                    return selection
                
                print("enter a number, from list above")
        
    def alert(self, issue):
        print("A: " + issue)

    def error(self, issue):
        print("E: " + issue)

    def account_format(self, account):
        return "{login}: {password}".format(*account)





import json
from os.path import exists

from passholder import *

filename = "storage"


def init_db():
    db = None
    global filename

    if exists(filename):
        tui.alert("default storage exists")
        if tui.choice("do you want to use this storage?"):
            resume = True
            while resume and not db:
                try:
                    db = DB.load(passphrase=tui.request_password("enter passphrase"))
                except errors.DecryptionFailed:
                    tui.error("decryption failed")
                    if not tui.choice("want to try again?"):
                        resume = False
                except FileNotFoundError:
                    tui.error("no encrypted data was provided")
                    resume = False
                except json.decoder.JSONDecodeError:
                    tui.error("json parsing error")
                except IsADirectoryError:
                    tui.error("it is a directory, must be a gpg file")
                    resume = False
            if db:
                tui.alert("storage loaded")
                return db, filename

    else:
        tui.alert("default storage not found")

    while not db:
        select = tui.select(["create new storage", "enter another filename", "quit"])
        if select == 1:
            if not exists(filename):
                db = DB.create()
                tui.alert("new storage created")
            else:
                tui.alert(f'storage "{filename}" exists')
                if tui.choice("do you want to overwrite?"):
                    db = DB.create()
                    tui.alert("new storage created")
        elif select == 2:
            filename = tui.request("enter filename")
            if exists(filename):
                resume = True
                while resume and not db:
                    try:
                        db = DB.load(filename, tui.request_password("enter password"))
                    except errors.DecryptionFailed:
                        tui.error("decryptuon failed")
                        if not tui.choice("want to try again? "):
                            resume = False
            else:
                tui.alert("storage not found")

        else:
            exit(0)

    tui.alert("storage loaded")
    return db, filename


def add():
    try:
        db.insert(tui.request("site"), tui.request("login"), tui.request("password"))
        tui.alert("account added")
    except errors.OverwriteError:
        tui.error("such account exists")


def search():
    try:
        site = tui.request("site")
        tui.alert(f"account found\nlogin: {db[site]['login']} password: {db[site]['password']}")
    except errors.AccountDoesNotExist:
        tui.error(f"account for {site} not found")


def delete():
    try:
        db.delete(tui.request("site"))
        tui.alert("account deleted")
    except errors.AccountDoesNotExist:
        tui.error("such account does not exist")


def save():
    db.dump(filename, tui.request_password("enter passphrase", repeat=True))
    tui.alert("changes saved")

def main():
    pass

running = True
tui = Terminal()
try:
    db, filename = init_db()
    while running:
       select = tui.select(["add new", "delete", "search", "save", "exit"])
       if select == 1:
           add()
       elif select == 2:
           delete()
       elif select == 3:
           search()
       elif select == 4:
           save()
       elif select == 5:
           running = False
           tui.alert("closing")
except KeyboardInterrupt:
    tui.alert("\n<C-c>: abort")
except EOFError:
    tui.alert("\n<C-d>: abort")
   



import json
import gnupg
from datetime import datetime
from passholder.errors import *


class DB:
    def __init__(self, db, gpg):
        self.__gpg = gpg
        self.__db = db


    def dump(self, filename, passphrase):
        with open(filename, "w") as f:
            data = str(json.dumps(self.__db, indent=2))
            encrypted_data = str(self.__gpg.encrypt(data, [], symmetric=True,
                                                    passphrase=passphrase))
            f.write(encrypted_data)

    def insert(self, site, login, password):
        if site in self.__db:
            raise OverwriteError(f"Account on {site} exists. Last modified \
                                 {self.__db[site]['date']}")
        else:
            self.__db[site] = { "login": login, "password": password,
                                "date": datetime.now().strftime("%d/%m/%Y %H:%M") }

    def update(self, site, login, password):
        if site in self.__db:
            self.__db[site] = { "login": login, "password": password,
                                "date": datetime.now().strftime("%d/%m/%Y %H:%M") }
        else:
            raise AccountDoesNotExist("Account to update does not exists")

    def search(self, pattern):
        pass

    def delete(self, site):
        if site in self.__db:
            del self.__db[site]
        else:
            raise AccountDoesNotExist("Account to delete does not exists")

    def __getitem__(self, site): # Usage: __db['vk.com']
        if site in self.__db:
            return self.__db[site]
        else:
            raise AccountDoesNotExist("Account to get does not exists")

    def __setitem__(self, site, value): # Usage: __db['vk.com'] = { 'login': 'login', 'password': 'password' }
       self.insert(site, value['login'], value['password'])

    def __delitem__(self, site): # Usage: del __db['vk.com']
        self.delete(site)


    @classmethod
    def create(cls):
        gpg = gnupg.GPG()
        gpg.encoding = "utf-8"

        db = {}

        return cls(db, gpg)

    @classmethod
    def load(cls, filename = "storage", passphrase = None):
        gpg = gnupg.GPG()
        gpg.encoding = "utf-8"
        db = {}
        with open(filename, "r") as f:
            encrypted_data = f.read()
            data = gpg.decrypt(encrypted_data,
                               passphrase=passphrase)

            if data.status == "decryption ok":
                if str(data):
                    db = json.loads(str(data))

            elif data.status == "decryption failed":
                raise DecryptionFailed("Something wrong with storage file")
            elif data.status == "no data was provided":
                raise FileNotFoundError("No encrypted data was provided")

        return cls(db, gpg)



class Account:
    def __init__(self, site, login, password):
        self.site     = site
        self.login    = login
        self.password = password

    def __str__(self):
        return "site: {site} \n login: {login} \n password: {password}".format(site     = self.site,
                                                                               login    = self.login,
                                                                               password = self.password)

    



from setuptools import setup, find_packages

reqs = [
    'getpass-asterisk==1.0.0',
    'python-gnupg==0.4.8',
]

setup(
    name='passholder',
    version='0.1.4',
    license='GPL-3.0',
    description = 'Password manager',
    url = 'https://github.com/aslanchek/Passholder',
    download_url = 'https://github.com/aslanchek/Passholder/archive/refs/tags/v0.1.4-alpha.tar.gz',
    packages=find_packages(),
    long_description='See github for long description.',
    install_requires=reqs,
    entry_points={
        'console_scripts':
        ['passholder = passholder.controller:main']
    },
    classifiers = [
        'Development Status :: 3 - Alpha',
        'Intended Audience :: End Users/Desktop',
        'Topic :: Database',
        'License :: OSI Approved :: GNU General Public License v3 (GPLv3)',
        'Programming Language :: Python :: 3.8',
    ]
)
